from debiaiServer.config.init_config import get_config
from debiaiServer.modules.dataProviders.DataProvider import DataProvider
from debiaiServer.modules.dataProviders.DataProviderException import DataProviderException
from debiaiServer.modules.dataProviders.pythonDataProvider.dataUtils import (
    pythonModuleUtils,
    projects,
    samples,
    selections,
    models,
    tree,
)

from debiaiServer.utils.utils import get_app_version

PYTHON_DATA_PROVIDER_ID = "Python module Data Provider"


# Wrappers
def project_must_exist(func):
    def wrapper(*args, **kwargs):
        if len(args) < 2:
            raise Exception("Project id must be provided as first argument")

        project_id = args[1]

        if not projects.project_exist(project_id):
            raise DataProviderException("Project " + project_id + " not found", 404)

        return func(*args, **kwargs)

    return wrapper


class PythonDataProvider(DataProvider):
    # Generic functions
    def __init__(self):
        pythonModuleUtils.init()
        nb_projects = len(projects.get_projects())
        print(
            "   Python module Data Provider initialized with "
            + str(nb_projects)
            + " projects"
        )

    @property
    def name(self):
        return PYTHON_DATA_PROVIDER_ID

    @property
    def type(self):
        return PYTHON_DATA_PROVIDER_ID

    def is_alive(self):
        return True

    def get_info(self):
        # Request method to get info on data Provider
        # return Object { version, dp_name, nb_Sample_max(to load)}
        return {
            "version": get_app_version(),
            "maxSampleIdByRequest": 10000,
            "maxSampleDataByRequest": 2000,
            "maxResultByRequest": 5000,
            "canDelete": {
                "projects": True,
                "selections": True,
                "models": True,
            },
        }

    # Projects
    def get_projects(self):
        # Request method to get projects overview
        # Return Arr[object{ id, name, nb_samples, nb_models, nb_selections,
        # update_time, creation_time}]
        return projects.get_projects()

    def create_project(self, name):
        # Check config
        config = get_config()
        creation_allowed = config["INTEGRATED_DATA_PROVIDER"]["allow_create_projects"]
        if not creation_allowed:
            raise DataProviderException("Project creation is not allowed", 403)

        # Project must not already exist
        if projects.project_exist(name):
            raise DataProviderException("Project already exists", 400)

        return projects.create_project(name, name)

    @project_must_exist
    def get_project(self, project_id):
        # Request method to get projects overview
        # Return object{ id, name, nb_samples, nb_models, nb_selections,
        # update_time, creation_time}

        project_base_info = projects.get_project(project_id)
        project_base_info["selections"] = selections.get_selections(project_id)
        project_base_info["resultStructure"] = projects.get_result_structure(project_id)
        project_base_info["models"] = models.get_models(project_id)
        return project_base_info

    @project_must_exist
    def delete_project(self, project_id):
        # Check config
        config = get_config()
        creation_allowed = config["INTEGRATED_DATA_PROVIDER"]["allow_delete_projects"]
        if not creation_allowed:
            raise DataProviderException("Project deletion is not allowed", 403)

        # Request method to delete project
        projects.delete_project(project_id)

    # Id list
    @project_must_exist
    def get_id_list(self, project_id, analysis, _from=None, _to=None):
        # Get id list
        # Return Arr[id]
        return samples.get_all_samples_id_list(project_id, _from, _to)

    @project_must_exist
    def get_samples(self, project_id, analysis, id_list):
        # Get full data from id list
        # Return object { id: [data]}
        return samples.get_data_from_sample_id_list(project_id, id_list)

    # Selections
    @project_must_exist
    def get_selections(self, project_id):
        # Get selections on project
        # Return arr[object{ id, name, creation_time, nb_samples}]
        return selections.get_selections(project_id)

    @project_must_exist
    def get_selection_id_list(self, project_id, selection_id):
        # Get selections id for a project
        # Return selection ID list
        return selections.get_selection_id_list(project_id, selection_id)

    @project_must_exist
    def create_selection(self, project_id, name, id_list, request_id=None):
        # Check config
        config = get_config()
        creation_allowed = config["INTEGRATED_DATA_PROVIDER"]["allow_create_selections"]
        if not creation_allowed:
            raise DataProviderException("Selection creation is not allowed", 403)

        # Selection creation
        return selections.create_selection(project_id, name, id_list, request_id)

    @project_must_exist
    def delete_selection(self, project_id, selection_id):
        # Check config
        config = get_config()
        deletion_allowed = config["INTEGRATED_DATA_PROVIDER"]["allow_delete_selections"]
        if not deletion_allowed:
            raise DataProviderException("Selection deletion is not allowed", 403)

        # Selection deletion
        return selections.delete_selection(project_id, selection_id)

    # Models
    @project_must_exist
    def get_models(self, project_id):
        return models.get_models(project_id)

    @project_must_exist
    def get_model_results_id_list(self, project_id, model_id):
        return models.get_model_id_list(project_id, model_id)

    @project_must_exist
    def get_model_results(self, project_id, model_id, id_list):
        return models.get_model_results(project_id, model_id, id_list)

    # Python module specific functions

    @project_must_exist
    def update_block_structure(self, project_id, blockStructure):
        # Check config
        config = get_config()
        creation_allowed = config["INTEGRATED_DATA_PROVIDER"]["allow_create_projects"]
        if not creation_allowed:
            raise DataProviderException("Project creation is not allowed", 403)

        # Update block structure
        projects.update_block_structure(project_id, blockStructure)

    @project_must_exist
    def add_block_tree(self, project_id, data):
        # Check config
        config = get_config()
        creation_allowed = config["INTEGRATED_DATA_PROVIDER"]["allow_insert_data"]
        if not creation_allowed:
            raise DataProviderException("Data insertion is not allowed", 403)

        # Insert data
        return tree.add_block_tree(project_id, data)

    @project_must_exist
    def update_results_structure(self, project_id, resultsStructure):
        # Check config
        config = get_config()
        creation_allowed = config["INTEGRATED_DATA_PROVIDER"]["allow_insert_results"]
        if not creation_allowed:
            raise DataProviderException("Results insertion is not allowed", 403)

        # TODO : check resultStructure (type and default type ==)
        existing_result_structure = projects.get_result_structure(project_id)
        if existing_result_structure is not None:
            raise DataProviderException(
                "project " + project_id + " already have a results structure", 403
            )

        projects.update_results_structure(project_id, resultsStructure)

    @project_must_exist
    def create_model(self, project_id, data):
        # Check config
        config = get_config()
        creation_allowed = config["INTEGRATED_DATA_PROVIDER"]["allow_create_models"]
        if not creation_allowed:
            raise DataProviderException("Model creation is not allowed", 403)

        models.create_model(
            project_id, data["name"], data["metadata"] if "metadata" in data else None
        )

    @project_must_exist
    def delete_model(self, project_id, model_id):
        # Check config
        config = get_config()
        deletion_allowed = config["INTEGRATED_DATA_PROVIDER"]["allow_delete_models"]
        if not deletion_allowed:
            raise DataProviderException("Model deletion is not allowed", 403)

        # Check if model exist
        if not models.model_exist(project_id, model_id):
            raise DataProviderException("Model does not exist", 404)

        models.delete_model(project_id, model_id)

    @project_must_exist
    def add_results_dict(self, project_id, model_id, data):
        # Check config
        config = get_config()
        creation_allowed = config["INTEGRATED_DATA_PROVIDER"]["allow_insert_results"]
        if not creation_allowed:
            raise DataProviderException("Results insertion is not allowed", 403)

        models.add_results_dict(project_id, model_id, data)



#############################################################################
# Imports
#############################################################################

import debiaiServer.modules.dataProviders.dataProviderManager as data_provider_manager
from debiaiServer.modules.dataProviders.DataProviderException import DataProviderException

#############################################################################
# MODELS Management
#############################################################################


def get_model_id_list(dataProviderId, projectId, modelId):
    """
    Get the list of models for a project
    """
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)
        return list(data_provider.get_model_results_id_list(projectId, modelId)), 200
    except DataProviderException as e:
        return e.message, e.status_code


def get_results(dataProviderId, projectId, modelId, data):
    """
    Get the model results from a sample list
    """
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)
        return (
            data_provider.get_model_results(projectId, modelId, data["sampleIds"]),
            200,
        )
    except DataProviderException as e:
        return e.message, e.status_code


def post_model(dataProviderId, projectId, data):
    # Create a new model
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)
        data_provider.create_model(projectId, data)
        return "model created", 200
    except DataProviderException as e:
        return e.message, e.status_code


def delete_model(dataProviderId, projectId, modelId):
    """
    Delete a model
    """
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)
        return data_provider.delete_model(projectId, modelId), 200
    except DataProviderException as e:
        return e.message, e.status_code



import uuid

#############################################################################
#
# Export type and method classes
#
# Those class are used to export data from a specific export type
#
#############################################################################


class ExportType:
    name = None
    parameters_definition = []
    export_method_class = None

    def to_dict(self):
        return {"name": self.name, "parameters": self.parameters_definition}


class ExportMethod:
    id = None
    type = None  # ExportType object
    name = None
    parameters = []

    deletable = False

    def __init__(self, type, name, parameters):
        self.id = uuid.uuid4().hex
        self.type = type
        self.name = name
        self.parameters = parameters

    def to_dict(self):
        return {
            "id": self.id,
            "type": self.type.name,
            "name": self.name,
            "parameters": self.parameters,
            "parameterNames": self.type.parameters_definition,
            "deletable": self.deletable,
        }



import os
from debiaiServer.modules.dataProviders.pythonDataProvider.dataUtils import (
    pythonModuleUtils,
    hash,
)

DATA_PATH = pythonModuleUtils.DATA_PATH


def getTagsIds(projectId):
    try:
        return os.listdir(DATA_PATH + projectId + "/tags")
    except FileNotFoundError:
        os.mkdir(DATA_PATH + projectId + "/tags")
        return []


def getTags(projectId):
    tagIds = getTagsIds(projectId)
    tags = []
    for tagId in tagIds:
        tag = getTagById(projectId, tagId)
        # Get the number of sample tagged
        tag["nbSamples"] = len(tag["tags"].keys())
        # remove the tag values
        tag.pop("tags", None)
        tags.append(tag)
    return tags


def getTagById(projectId, tagId):
    if tagId not in getTagsIds(projectId):
        return None

    return pythonModuleUtils.readJsonFile(
        DATA_PATH + projectId + "/tags/" + tagId + "/info.json"
    )


def getTagByName(projectId, tagName):
    for tagId in getTagsIds(projectId):
        tag = getTagById(projectId, tagId)
        if tag["name"] == tagName:
            return tag
    return None


def updateTag(projectId, tagName, tagHash):
    # TODO change to tagId
    # ParametersCheck
    projectHashMap = hash.getHashmap(projectId)

    for sampleHash in tagHash.keys():
        if sampleHash not in projectHashMap:
            return "SampleHash not found in the project samples", 404

    tag = getTagByName(projectId, tagName)
    if tag:
        # Update tag
        for sampleHash in tagHash.keys():
            if tagHash[sampleHash] == 0:
                tag["tags"].pop(sampleHash, None)
            else:
                tag["tags"][sampleHash] = tagHash[sampleHash]

        tag["updateDate"] = pythonModuleUtils.timeNow()
        pythonModuleUtils.writeJsonFile(
            DATA_PATH + projectId + "/tags/" + tag["id"] + "/info.json", tag
        )
        return tag, 200
    else:
        # Create tag
        # tag ID
        tagId = pythonModuleUtils.clean_filename(tagName)
        if len(tagId) == 0:
            tagId = pythonModuleUtils.timeNow()

        nbTag = 1
        while tagId in getTagsIds(projectId):
            tagId = pythonModuleUtils.clean_filename(tagName) + "_" + str(nbTag)
            nbTag += 1

        # Save tag
        os.mkdir(DATA_PATH + projectId + "/tags/" + tagId)
        now = pythonModuleUtils.timeNow()
        tagInfo = {
            "id": tagId,
            "name": tagName,
            "tags": tagHash,
            "creationDate": now,
            "updateDate": now,
        }

        pythonModuleUtils.writeJsonFile(
            DATA_PATH + projectId + "/tags/" + tagId + "/info.json", tagInfo
        )

        return tagInfo, 200


def deleteTag(projectId, tagId):
    pythonModuleUtils.deleteDir(DATA_PATH + projectId + "/tags/" + tagId)


def getSamplesHash(projectId, tagId, tagValue):
    tag = getTagById(projectId, tagId)
    hash = []
    for sampleHash in tag["tags"].keys():
        if tag["tags"][sampleHash] == tagValue:
            hash.append(sampleHash)

    return hash



import time
import yaml
from yaml.loader import SafeLoader
from urllib.parse import urlparse


def get_app_version():
    # Read the version from the API YAML file
    try:
        with open("debiaiServer/swagger.yaml") as f:
            data = yaml.load(f, Loader=SafeLoader)
            return data["info"]["version"]
    except Exception as e:
        print(e)
        return "?.?.?"


# Date
def timeNow():
    return time.time() * 1000


# Url
def is_url_valid(url):
    try:
        result = urlparse(url)
        return all([result.scheme, result.netloc])
    except Exception:
        return False


# Name
def is_valid_name(name):
    # /, &, | are not allowed in data-providers & algo-providers names
    if (
        "/" in name
        or "&" in name
        or "|" in name
        or len(name) == 0
        or len(name) > 50
        or name[0] == " "
        or name[-1] == " "
    ):
        return False

    return True



import debiaiServer.modules.dataProviders.webDataProvider.http.api as api

#
# UseCase folder role is the middleware between class methods and http requests
# It's used to make all changes in data we took from DP and send it back to
# the class/controller
#


def get_project_id_list(url, cache, id_project, analysis, _from=None, _to=None):
    id_list = cache.get_id_list(id_project, _from, _to)

    if id_list is None:
        id_list = api.get_id_list(url, id_project, analysis, _from, _to)
        cache.set_id_list(id_project, id_list, _from, _to)

    return id_list


def get_project_samples(url, id_project, analysis, id_list):
    data = api.get_samples(url, id_project, analysis, id_list)

    return data



from debiaiServer.modules.dataProviders import (
    dataProviderManager,
)
from debiaiServer.modules.exportMethods import (
    exportUtils,
)
from debiaiServer.modules.algoProviders import (
    algoProvidersManager,
)
from debiaiServer.utils.widgetConfigurations import (
    widgetConfigurations as widgetConfUtils,
)
from debiaiServer.utils.layouts import (
    layouts as layoutsUtils,
)
from debiaiServer.config import (
    init_config as config,
)


def init():
    # Init config file
    config.init_config()

    # Init data providers
    dataProviderManager.setup_data_providers()

    # Init AlgoProviders
    algoProvidersManager.setup_algo_providers()

    # Init export methods
    exportUtils.load_export_methods()

    # Init widget configurations
    widgetConfUtils.setup_widget_configurations()

    # Init layouts
    layoutsUtils.setup_layouts()



from debiaiServer.modules.dataProviders.DataProvider import DataProvider
from debiaiServer.modules.dataProviders.webDataProvider.useCases.data import (
    get_project_id_list,
    get_project_samples,
)
from debiaiServer.modules.dataProviders.webDataProvider.useCases.projects import (
    get_all_projects_from_data_provider,
    get_single_project_from_data_provider,
    delete_project,
)
from debiaiServer.modules.dataProviders.webDataProvider.useCases.models import (
    get_model_results,
    get_models_info,
    get_model_result_id,
    delete_model,
)
import debiaiServer.modules.dataProviders.webDataProvider.useCases.selections as useCaseSelections  # noqa
from debiaiServer.modules.dataProviders.webDataProvider.http.api import get_info, get_status
from debiaiServer.modules.dataProviders.webDataProvider.cache.cache import Cache


# WebDataProvider class, allow to get data from a web data-provider
class WebDataProvider(DataProvider):
    def __init__(self, url, name):
        self.url = url
        self._name = name
        self.alive = None

        # Init cache
        self.cache = Cache()

    @property
    def name(self):
        return self._name

    @property
    def type(self):
        return "Web"

    # Todo api call Info (new info)
    def is_alive(self):
        self.alive = True if get_status(self.url) is True else False
        return self.alive

    def get_info(self):
        return get_info(self.url)

    # ==== Projects ====
    def get_projects(self):
        # Request method to get projects overview
        # Return Arr[object{ id, name, nb_samples, nb_models, nb_selections,
        # update_time, creation_time}]
        return get_all_projects_from_data_provider(self.url, self.name)

    def get_project(self, id_project):
        # Request method to get projects overview
        # Return object{ id, name, nb_samples, nb_models, nb_selections,
        # update_time, creation_time}
        return get_single_project_from_data_provider(self.url, self.name, id_project)

    def delete_project(self, project_id):
        return delete_project(self.url, project_id)

    def get_id_list(self, project_id, analysis, _from=None, _to=None):
        # http Request on dp to get id list
        # Return Arr[id]
        return get_project_id_list(
            self.url, self.cache, project_id, analysis, _from, _to
        )

    def get_samples(self, project_id, analysis, id_list):
        # http Request get full sample
        # Return object { id: [data]}
        return get_project_samples(self.url, project_id, analysis, id_list)

    # ==== Selections ====
    def get_selections(self, project_id):
        # Get selections on project
        # Return arr[object{ id, name, creation_time, nb_samples}]
        return useCaseSelections.get_project_selections(self.url, project_id)

    def get_selection_id_list(self, project_id, selection_id):
        return useCaseSelections.get_id_list_from_selection(
            self.url, self.cache, project_id, selection_id
        )

    def create_selection(self, project_id, name, id_list, request_id=None):
        return useCaseSelections.create_selection(
            self.url, project_id, name, id_list, request_id
        )

    def delete_selection(self, project_id, selection_id):
        return useCaseSelections.delete_selection(self.url, project_id, selection_id)

    # ==== Models ====
    def get_models(self, project_id):
        return get_models_info(self.url, project_id)

    def get_model_results_id_list(self, project_id, model_id):
        return get_model_result_id(self.url, self.cache, project_id, model_id)

    def get_model_results(self, project_id, model_id, sample_list):
        return get_model_results(self.url, project_id, model_id, sample_list)

    def delete_model(self, project_id, model_id):
        return delete_model(self.url, project_id, model_id)



import ujson as json
import os

from debiaiServer.modules.dataProviders.pythonDataProvider.dataUtils import (
    pythonModuleUtils,
    projects,
    models,
    hash,
)

DATA_PATH = pythonModuleUtils.DATA_PATH
DATA_TYPES = pythonModuleUtils.DATA_TYPES

# def getFirstLevelBlock(projectId, blockId):
#     blockList = os.listdir(DATA_PATH + projectId + '/blocks')

#     if (blockId not in blockList):
#         return -1

#     with open(DATA_PATH + projectId + '/blocks/' + blockId + '/info.json')\
#             as json_file:
#         data = json.load(json_file)

#     return data


# def getBlockTree(projectId, path, depth):

#     childrenBlockNames = utils.listDir(path)
#     childrenInfo = []

#     if depth > 0:
#         # Get the children blocks info
#         for name in childrenBlockNames:
#             childrenInfo.append(getBlockTree(
#                 projectId, path + name + "/", depth - 1))

#     with open(path + 'info.json') as json_file:
#         data = json.load(json_file)
#         data['childrenInfoList'] = childrenInfo

#     return data


def add_block_tree(projectId, data):
    # Loading project block info
    bli = projects.get_project_block_level_info(projectId)

    # going through the tree to check for error, store the block to add
    blockToAdd = []

    try:
        for block in data["blockTree"]:
            addBlockTree(projectId, block, bli, blockToAdd, 0, "")
    except KeyError as e:
        print(str(e))
        print("badInputTree")
        return str(e), 403

    if len(blockToAdd) == 0:
        return "No block added", 201

    # Store the blocks and the hash map
    sampleLevel = len(bli) - 1
    hashToSave = {}
    for block in blockToAdd:
        if block["level"] == sampleLevel:
            # Sample level, creating hash
            sampleHash = hash.hash(block["path"])
            block["id"] = sampleHash
            hashToSave[sampleHash] = block["path"]

        addBlock(projectId, block)

    # Save hashmap
    hash.addToSampleHashmap(projectId, hashToSave)

    projects.update_project(projectId)
    return str(len(blockToAdd)) + " added blocks"


def getBlockInfo(blockLevel, blockInfo):
    """
    Convert the block info to fill the sampleInfo list with a colonName:value dict
    """
    print("get block info")
    print(blockLevel)
    print(blockInfo)
    ret = {}
    ret[blockLevel["name"]] = blockInfo["name"]

    for dataType in pythonModuleUtils.DATA_TYPES:
        if dataType in blockLevel:
            for i, column in enumerate(blockLevel[dataType]):
                ret[column["name"]] = blockInfo[dataType][i]

    return ret


def getBlockTreeFromSamples(projectId, samples: list):
    blocksData = []
    addedBlocks = []

    for samplePath in samples:
        try:
            sampleBlocksData, endLevel = __getBlockTreeFromSample(
                projectId, samplePath, addedBlocks
            )

            if endLevel == 0:
                # root block, should be here after added the first sample
                blocksData.append(sampleBlocksData)
            else:
                # Not a root block, we need to find where to insert it
                # First, find the root
                cur = next(
                    block
                    for block in blocksData
                    if block["path"] in sampleBlocksData["path"]
                )

                # Then, find the level
                for i in range(0, endLevel - 1):
                    cur = next(
                        child
                        for child in cur["childrenInfoList"]
                        if child["path"] in sampleBlocksData["path"]
                    )

                cur["childrenInfoList"].append(sampleBlocksData)
        except StopIteration:
            # TODO : Find why this happens of certain projects
            print("Warning, the sample " + samplePath + " doesn't have a root block")

    return blocksData


def __getBlockTreeFromSample(projectId, blockPath, addedBlocks):
    """
    Go from the bottom to the top of a tree
    if at the top or, if block already added, return

    """
    # Add the block we are in
    addedBlocks.append(blockPath)

    with open(
        DATA_PATH + projectId + "/blocks/" + blockPath + "/info.json"
    ) as sampleData:
        info = json.load(sampleData)

    if info["level"] == 0:
        # Top of the tree, end of the recursively
        return info, 0

    if info["parentPath"] in addedBlocks:
        # The block to the top is already added, there is no need to go further
        return info, info["level"]

    # Climbing up the tree of one level
    parentInfo, endLevel = __getBlockTreeFromSample(
        projectId, info["parentPath"], addedBlocks
    )

    # Let's add our info to the parents
    cur = parentInfo

    for i in range(endLevel, info["level"] - 1):
        cur = cur["childrenInfoList"][0]
    cur["childrenInfoList"] = [info]

    return parentInfo, endLevel


def addResultsToTree(projectId, tree: list, modelIds: list, commonOnly: bool) -> dict:
    """
    Add, in the tree samples, the results from a model id list
    """

    # Load all the model results
    modelResults = {}
    for modelId in modelIds:
        modelResults[modelId] = models.getModelResults(projectId, modelId)

    # Get the project block structure
    proBs = projects.get_project_block_level_info(projectId)
    sampleLevel = len(proBs) - 1

    # Add in the samples the results
    for rootBlock in tree:
        __addResultsToABlock(rootBlock, modelResults, sampleLevel, commonOnly)

    return tree


def __addResultsToABlock(block, modelResults, sampleLevel, commonOnly):
    if block["level"] == sampleLevel:
        # Adding the results to the sample
        block["results"] = {}
        for modelId in modelResults:
            # If no more than 1 model and commonOnly, no need to check if
            # sample exist in tree
            if (
                commonOnly
                or len(modelResults) == 1
                or block["path"] in modelResults[modelId]
            ):
                block["results"][modelId] = modelResults[modelId][block["path"]]
        return

    for child in block["childrenInfoList"]:
        __addResultsToABlock(child, modelResults, sampleLevel, commonOnly)


# Add samples to a tree
def addBlockTree(projectId, block, blockLevelInfo, blockToAdd, level, parentPath):
    __checkBlockCompliant(block, level, blockLevelInfo)

    # check if block exist
    data = findBlockInfo(projectId, parentPath + block["name"])
    if data is None:
        # BLock doesn't exist
        blockToAdd.append(__createBlock(projectId, block, level, parentPath))

    path = parentPath + block["name"] + "/"

    if level < len(blockLevelInfo) - 1:
        for child in block["childrenInfoList"]:
            addBlockTree(projectId, child, blockLevelInfo, blockToAdd, level + 1, path)


def findBlockInfo(projectId, blockPath):
    curPath = DATA_PATH + projectId + "/blocks/" + blockPath

    if not os.path.isdir(curPath):
        return None

    with open(curPath + "/info.json", "r") as json_file:
        data = json.load(json_file)

    return data


def __checkBlockCompliant(block, level, blockLevelInfo):
    # Check if a block is correct (name exist, levelInfo coherence, etc)

    if "name" not in block:
        raise KeyError("A block at level " + str(level) + " is missing his name")

    # TODO check name valid (no / & .)

    if "childrenInfoList" not in block and level < (len(blockLevelInfo) - 1):
        raise KeyError(
            "Block : " + block["name"] + " has no childrenInfoList properties"
        )

    if level < len(blockLevelInfo) - 1 and len(block["childrenInfoList"]) == 0:
        raise KeyError(
            "Block : "
            + block["name"]
            + " has no child block, the tree need to be complete"
        )

    levelInfo = blockLevelInfo[level]

    for type_ in DATA_TYPES:
        if type_ in levelInfo and len(levelInfo[type_]) > 0:
            if type_ not in block:
                raise KeyError(
                    "At least one value of type "
                    + type_
                    + " is required in the block : "
                    + levelInfo["name"]
                )

            if len(block[type_]) != len(levelInfo[type_]):
                raise KeyError(
                    "Exactly "
                    + str(len(levelInfo[type_]))
                    + " "
                    + type_
                    + " required in the block : "
                    + levelInfo["name"]
                )

            # TODO Implement column default

            for i, col in enumerate(levelInfo[type_]):
                if col["type"] == "integer" and type(block[type_][i]) is str:
                    raise KeyError(
                        "Col "
                        + col["name"]
                        + " require an integer in the block : "
                        + levelInfo["name"]
                    )


def __createBlock(projectId, block, level, parentPath):
    blockPath = parentPath + block["name"] + "/"

    debiaiBlock = {
        "id": block["name"],
        "name": block["name"],
        "path": blockPath,
        "parentPath": parentPath,
        "level": level,
        # "creationDate": str(date.today()),
        # "updateDate": str(date.today()),
        # "version": "0.0.0",
        # "metaDataList": {},
    }

    for type_ in DATA_TYPES:
        if type_ in block:
            debiaiBlock[type_] = block[type_]

    return debiaiBlock


def addBlock(projectId, block):
    # create the block folder and his info.json file
    try:
        os.mkdir(DATA_PATH + projectId + "/blocks/" + block["path"])
        pythonModuleUtils.writeJsonFile(
            DATA_PATH + projectId + "/blocks/" + block["path"] + "/info.json", block
        )
    except FileExistsError:
        print(
            "Warning : The block "
            + block["path"]
            + " already exist, this is not supposed to append"
        )



from debiaiServer.modules.algoProviders.integratedAlgoProvider.utils import (
    get_input_from_inputs,
)

# from ..utils import get_input_from_inputs

# This algorithm is a simple classification metric calculator
# It takes a list of values corresponding to the ground truth
# and a list of values corresponding to the predictions

# It returns a list of True/False values corresponding to
# whether the prediction is correct or not

# It also returns the accuracy percentage and the number of correct predictions

# Technical details (must respect the algo-api format):
algorithm_description = {
    "name": "Classification Metric",
    "description": """Calculates the classification error according \
to the ground truth and the predictions""",
    "author": "DebiAI",
    "version": "1.0.0",
    "creationDate": "2023-10-30",
    "tags": ["metrics", "classification"],
    "inputs": [
        {
            "name": "Ground truth",
            "description": "List of ground truth values",
            "type": "array",
            "arrayType": "text",
        },
        {
            "name": "Predictions",
            "description": "List of predictions, must have the same \
length as the ground truth list",
            "type": "array",
            "arrayType": "text",
        },
    ],
    "outputs": [
        {
            "name": "Binary error",
            "description": "Classification metric of the input list, \
False if GDT == PRED, True otherwise",
            "type": "array",
            "arrayType": "boolean",
        },
        {
            "name": "Binary success",
            "description": "Classification metric of the input list, \
True if GDT == PRED, False otherwise",
            "type": "array",
            "arrayType": "boolean",
        },
        {
            "name": "Accuracy",
            "description": "Percentage of correct predictions",
            "type": "number",
        },
        {
            "name": "Number of correct predictions",
            "type": "number",
        },
    ],
}


def get_algorithm_details():
    return algorithm_description


def use_algorithm(inputs):
    # Get inputs
    gdt = get_input_from_inputs(inputs, "Ground truth", "array")
    predictions = get_input_from_inputs(inputs, "Predictions", "array")

    # Check inputs
    if len(gdt) != len(predictions):
        raise TypeError("Ground truth and predictions must have the same length")

    # Calculate classification metric
    binary_error = [None] * len(gdt)
    binary_success = [None] * len(gdt)
    nb_correct_predictions = 0
    for i in range(len(gdt)):
        if gdt[i] == predictions[i]:
            nb_correct_predictions += 1
            binary_error[i] = False
            binary_success[i] = True
        else:
            binary_error[i] = True
            binary_success[i] = False

    # Calculate accuracy
    accuracy = nb_correct_predictions / len(binary_success)

    # Return outputs
    return [
        {"name": "Binary error", "value": binary_error},
        {"name": "Binary success", "value": binary_success},
        {"name": "Accuracy", "value": accuracy},
        {"name": "Number of correct predictions", "value": nb_correct_predictions},
    ]



from debiaiServer.modules.dataProviders.DataProviderException import DataProviderException
import debiaiServer.modules.dataProviders.dataProviderManager as data_provider_manager


# Project
def post_project(data):
    # Ask a data provider to create a project
    dataProviderId = "Python module Data Provider"
    projectName = data["projectName"]

    # Check project name
    if len(projectName) > 100:
        return "Project name too long", 400

    # Create project
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)

        project = data_provider.create_project(projectName)

        # Adding data provider id to project
        project["dataProviderId"] = dataProviderId

        return project, 200
    except DataProviderException as e:
        return e.message, e.status_code


# Block level
def post_block_levels(dataProviderId, projectId, block_levels):
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)
        data_provider.update_block_structure(projectId, block_levels)
        return block_levels, 200
    except DataProviderException as e:
        return e.message, e.status_code


# Expected_results
def post_resultsStructure(dataProviderId, projectId, resultStructure):
    # Add the expected results structure
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)
        data_provider.update_results_structure(projectId, resultStructure)
        return resultStructure, 200
    except DataProviderException as e:
        return e.message, e.status_code


def post_block_tree(dataProviderId, projectId, data):
    # Add data to a project from a tree
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)
        return data_provider.add_block_tree(projectId, data), 200
    except DataProviderException as e:
        return e.message, e.status_code


# Add model results
def add_results_dict(dataProviderId, projectId, modelId, data):
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)
        data_provider.add_results_dict(projectId, modelId, data)
        return "Results added", 200
    except DataProviderException as e:
        return e.message, e.status_code



from kafka import KafkaProducer
from debiaiServer.modules.exportMethods.exportClass import ExportType, ExportMethod
import json

#############################################################################
#
# Kafka export method
#
# Connect to a kafka server on init and send data to a topic
#
#############################################################################


class KafkaExportType(ExportType):
    def __init__(self):
        super().__init__()

        self.name = "kafka"
        # Expected parameters: [server, topic]
        self.parameters_definition = ["server", "topic"]

        self.export_method_class = KafkaExportMethod


class KafkaExportMethod(ExportMethod):
    up = False

    def __init__(self, name, parameters):
        super().__init__(KafkaExportType(), name, parameters)

        # Expected parameters: [server, topic]
        # Check parameters
        if len(parameters) != 2:
            raise Exception(
                "Kafka export type requires 2 parameters : server and topic"
            )

        # Create producer
        self.server = parameters[0]
        self.topic = parameters[1]

        # Create Kafka producer
        try:
            self.producer = KafkaProducer(
                bootstrap_servers=self.server,
                value_serializer=lambda v: json.dumps(v).encode("utf-8"),
            )
            self.up = True
        except Exception as e:
            print("Kafka producer creation failed : " + str(e))
            print("server : '" + self.server + "'")
            raise Exception(
                "Kafka producer creation on server '"
                + self.server
                + "' failed with error : "
                + str(e)
            )

    def export(self, data):
        print("Kafka export method : Sending data to kafka", self.server, self.topic)
        print(data)

        if not self.up:
            raise Exception("Kafka producer is not up")

        try:
            print(self.producer.send(self.topic, data))
            print("Kafka export method : Data sent")
        except Exception as e:
            print("Kafka export method : Error sending data to kafka", e)
            raise "Kafka export method : Error sending data to kafka"



import connexion
import os
import requests
import webbrowser
import psutil
from termcolor import colored
from threading import Timer
from flask_cors import CORS
from flask import send_from_directory, request, Response
from debiaiServer.init import init
from debiaiServer.utils.utils import get_app_version
from debiaiServer.config.init_config import DEBUG_COLOR

DEV_FRONTEND_URL = "http://localhost:8080/"
PORT = 3000
app = connexion.App(__name__)
app.add_api("swagger.yaml", strict_validation=True)
CORS(app.app)


def send_frontend(path):
    if path == "/":
        path = "index.html"

    # If production, use the index.html from the dist folder
    env = os.getenv("FLASK_ENV", "production")
    debug_mode = env == "production"
    if debug_mode:
        return send_from_directory("dist", path)

    # In development, redirect to the DEV_FRONTEND_URL
    else:
        if request.method == "GET":
            try:
                resp = requests.get(f"{DEV_FRONTEND_URL}{path}")
                excluded_headers = [
                    "content-encoding",
                    "content-length",
                    "transfer-encoding",
                    "connection",
                ]
                headers = [
                    (name, value)
                    for (name, value) in resp.raw.headers.items()
                    if name.lower() not in excluded_headers
                ]
                response = Response(resp.content, resp.status_code, headers)
                return response
            except (requests.exceptions.ConnectionError, requests.exceptions.Timeout):
                return (
                    "You are in a development environment and the DebiAI frontend"
                    + "is not available at the url : "
                    + DEV_FRONTEND_URL,
                    503,
                )
        else:
            print("Unexpected request method")


def create_app():
    # For serving the dashboard
    @app.route("/")
    def send_index():
        return send_frontend("/")

    # For serving the dashboard assets
    @app.route("/<path:path>")
    def send_supporting_elements(path):
        return send_frontend(path)

    return app


def is_browser_open():
    """Check if a browser process is running."""
    browser_keywords = ["chrome", "firefox", "safari", "edge", "opera"]
    for proc in psutil.process_iter(["pid", "name"]):
        for keyword in browser_keywords:
            if keyword in proc.info["name"].lower():
                return True
    return False


def open_browser():
    url = f"http://localhost:{PORT}"
    if is_browser_open():
        webbrowser.open_new_tab(url)
    else:
        webbrowser.open(url)


def start_server(reloader=True):
    # Run DebiAI init
    print("================= DebiAI " + get_app_version() + " ====================")
    init()
    print("======================== RUN =======================")
    print(
        "   DebiAI is available at "
        + colored("http://localhost:" + str(PORT), DEBUG_COLOR)
    )
    app = create_app()
    app.run(port=PORT, debug=True, use_reloader=reloader)



from debiaiServer.config.init_config import get_config
import debiaiServer.modules.dataProviders.dataProviderManager as data_provider_manager
from debiaiServer.modules.dataProviders.DataProviderException import DataProviderException
import time

from debiaiServer.modules.exportMethods.methods.kafkaUtils import KafkaExportType
from debiaiServer.modules.exportMethods.methods.postUtils import PostExportType

#############################################################################
#
# Export utils
#
# DebiAI allows to export data or selections to other services with
# different methods.
# This utils load and store the methods for all the projects
#
#############################################################################

# The export types are the different types of export methods that we can create
# They are used to create the export methods
export_types = [KafkaExportType(), PostExportType()]

# The export methods are the different methods created from the types
# They we can used to export data
export_methods = []


# Export types
def get_export_type(typeName):
    return [type for type in export_types if type.name == typeName][0]


def type_exist(typeName):
    return typeName in [type.name for type in export_types]


# Export utils
def get_export_methods():
    # Return all the export methods as a list of dictionaries
    return [method.to_dict() for method in export_methods]


def get_export_method(methodId):
    # Check the method id
    if not method_exist(methodId):
        raise Exception("Export method " + methodId + " not found")

    return [method for method in export_methods if method.id == methodId][0]


def method_exist(methodId):
    return methodId in [method.id for method in export_methods]


def load_export_methods():
    global export_methods
    print("================== EXPORT METHODS ==================")

    # Load the export methods from the config file
    config = get_config()

    if "EXPORT_METHODS_LIST" in config:
        print(" - Loading export methods from config file")
        config_export_methods = config["EXPORT_METHODS_LIST"]

        for method in config_export_methods:
            print(
                "     Adding method " + method, "[", config_export_methods[method], "]"
            )

            try:
                parameters = config_export_methods[method].split(",")
                if len(parameters) == 0:
                    raise "method " + method + " has no parameters, aborting"

                # Trim parameters
                for i in range(len(parameters)):
                    parameters[i] = "".join(parameters[i].rstrip().lstrip())

                export_type_name = parameters[0]
                export_method = create_export_method(
                    method, export_type_name, parameters[1:]
                )

                if config["EXPORT_METHODS_CONFIG"]["deletion"]:
                    # The export method created from the config file are deletable
                    export_method.deletable = True

                export_methods.append(export_method)
            except Exception as e:
                print("Error while configuring method " + method + ": " + str(e))

    if len(export_methods) == 0:
        print("   No export method configured")


def add_export_method(data):
    # Check the method type
    if not type_exist(data["type"]):
        raise Exception("Method type " + data["type"] + " not found")

    export_method = create_export_method(data["name"], data["type"], data["parameters"])

    config = get_config()
    if config["EXPORT_METHODS_CONFIG"]["deletion"]:
        # The export method created from the config file are deletable
        export_method.deletable = True

    export_methods.append(export_method)

    return export_method.to_dict()


def create_export_method(name, type, parameters):
    # Check the method type
    if not type_exist(type):
        raise Exception(
            "Export type '"
            + type
            + "' isn't supported, only "
            + str([type.name for type in export_types])
            + " are supported"
        )

    # Get the export type
    export_type = get_export_type(type)

    # Create the method
    return export_type.export_method_class(name, parameters)


def delete_export_method(method_id):
    global export_methods

    # Check the method id
    if not method_exist(method_id):
        raise Exception("The export method wasn't found")

    # Delete the method
    export_methods = [method for method in export_methods if method.id != method_id]
    return "method " + method_id + " deleted"


# Export data
def exportSelection(dataProviderId, projectId, data):
    method_id = data["exportMethodId"]

    # Check the method id
    if not method_exist(method_id):
        raise Exception("method " + method_id + " not found")

    export_method = get_export_method(method_id)

    # Creation of the data selection to export
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)
        project = data_provider.get_project(projectId)
    except DataProviderException as e:
        return e.message, e.status_code

    id_list = []

    for id in data["sampleHashList"]:
        id_list.append({"id": id})

    data_to_export = {
        "origin": "DebiAI",
        "type": "selection",
        "projectId": projectId,
        "data_provider_id": dataProviderId,
        "selection_name": data["selectionName"],
        "date": time.time(),
        "sample_ids": id_list,
    }

    # Project name
    if "name" in project:
        data_to_export["project_name"] = project["name"]

    # Annotation extra value
    if "annotationValue" in data and data["annotationValue"] != "":
        data_to_export["value"] = data["annotationValue"]

    # Export the data
    export_method.export(data_to_export)

    return "data exported"


def exportData(method_id, data):
    # Check the method id
    if not method_exist(method_id):
        raise Exception("method " + method_id + " not found")

    export_method = get_export_method(method_id)

    # Export the data
    export_method.export(data)

    return "data exported"



#############################################################################
# Imports
#############################################################################
from debiaiServer.config.init_config import get_config
from debiaiServer.modules.dataProviders.webDataProvider.WebDataProvider import (
    WebDataProvider,
)
from debiaiServer.utils.utils import is_url_valid
import debiaiServer.modules.dataProviders.dataProviderManager as data_provider_manager
from debiaiServer.modules.dataProviders.DataProviderException import DataProviderException

#############################################################################
# Data Providers Management
#############################################################################


def get_data_providers():
    data_provider_list = data_provider_manager.get_data_provider_list()
    providers_formatted = []
    for data_provider in data_provider_list:
        data = {}
        if data_provider.type != "Python module Data Provider":
            data["url"] = data_provider.url
            data["status"] = data_provider.is_alive()

        data["name"] = data_provider.name
        data["type"] = data_provider.type

        providers_formatted.append(data)

    return providers_formatted, 200


def get_data_provider_info(dataProviderId):
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)
        info = data_provider.get_info()

        return info, 200
    except DataProviderException as e:
        return e.message, e.status_code


def post_data_providers(data):
    # Check if we are allowed to add data providers from the config file
    config = get_config()
    creation_allowed = config["DATA_PROVIDERS_CONFIG"]["creation"]
    if not creation_allowed:
        return "Data provider creation is not allowed", 403

    # Check if data provider already exists
    if data_provider_manager.data_provider_exists(data["name"]):
        return "Data provider already exists", 400

    # Check if data provider name is valid
    if not data_provider_manager.is_valid_name(data["name"]):
        return "Invalid data provider name", 400

    try:
        # Add data provider
        if data["type"].lower() == "web":
            # Check if url is valid
            if "url" not in data:
                return "A url must be provided", 400

            if not is_url_valid(data["url"]):
                return "Invalid url", 400

            data_provider_manager.add(WebDataProvider(data["url"], data["name"]))
        else:
            return "Invalid data provider type, valid types are: Web", 400

        return None, 204
    except DataProviderException as e:
        return e.message, e.status_code


def delete_data_providers(dataProviderId):
    # Check if we are allowed to add data providers from the config file
    config = get_config()
    deletion_allowed = config["DATA_PROVIDERS_CONFIG"]["deletion"]
    if not deletion_allowed:
        return "Data provider deletion is not allowed", 403

    # Delete data provider
    try:
        data_provider_manager.delete(dataProviderId)
        return None, 204
    except DataProviderException as e:
        return e.message, e.status_code



# Class for AlgoProvider
import requests
import json
from debiaiServer.modules.algoProviders.AlgoProviderException import AlgoProviderException


class AlgoProvider:
    def __init__(self, url, name):
        self.url = url
        self.name = name
        self.alive = False

    def is_alive(self):
        # Try to load algorithms
        self.alive = True if self.get_algorithms() is not None else False
        return self.alive

    def get_algorithms(self):
        try:
            r = requests.get(self.url + "/algorithms")
            return get_http_response(r)
        except (
            requests.exceptions.ConnectionError,
            requests.exceptions.Timeout,
            requests.exceptions.HTTPError,
        ):
            return None

        except Exception as e:
            print("Error in get_algorithms")
            print(e)
            return None

    def to_json(self):
        algorithms = None
        if self.is_alive():
            algorithms = self.get_algorithms()

        return {
            "name": self.name,
            "url": self.url,
            "status": self.alive,
            "algorithms": algorithms,
        }

    def use_algorithm(self, algorithm_id, data):
        try:
            print("Using algoProvider: " + self.url)
            print("Using algorithm: " + algorithm_id)
            r = requests.post(
                self.url + "/algorithms/" + algorithm_id + "/run", json=data
            )
            if r.raise_for_status() is None:
                return get_valid_response(r)
        except (
            requests.exceptions.ConnectionError,
            requests.exceptions.Timeout,
        ) as e:
            print("The algoProvider is not reachable")
            print(e)
            raise AlgoProviderException("AlgoProvider not reachable", 500)
        except requests.exceptions.HTTPError as e:
            print("The algoProvider returned an error")
            print(e)
            print(e.response.text)
            print(e.response.json())

            if "detail" in e.response.json():
                raise AlgoProviderException(e.response.json()["detail"], 400)

            if e.response.status_code == 500:
                raise AlgoProviderException(
                    "AlgoProvider internal server error: " + str(e), 500
                )
            elif e.response.status_code == 400:
                raise AlgoProviderException(e.response.text, 400)

            elif e.response.status_code == 404:
                raise AlgoProviderException(
                    "The algoProvider may not have this algorithm, " + e.response.text,
                    404,
                )
            else:
                raise AlgoProviderException(str(e), 400)


# ==== Utils ====
def get_http_response(response):
    try:
        if response.raise_for_status() is None:
            return get_valid_response(response)
    except requests.exceptions.HTTPError:
        return get_error_response(response)


def get_valid_response(response):
    if response.status_code == 204:
        return True
    try:
        return response.json()
    except json.decoder.JSONDecodeError:
        return


def get_error_response(response):
    if response.status_code == 500:
        raise AlgoProviderException("AlgoProvider unexpected Error", 500)

    raise AlgoProviderException(response.text, response.status_code)



def get_input_from_inputs(
    inputs, input_name, expected_input_type=None, expected_list_type=None
):
    # Get the input from of the inputs list from a given name
    # Check the type and the subtype if needed

    for i, input in enumerate(inputs):
        if "name" not in input:
            raise TypeError("Input n°{} has no name".format(i))

        if "value" not in input:
            raise TypeError("Input {} has no value".format(input["name"]))

        if input["name"] == input_name:
            # Check the type
            if expected_input_type == "number":
                if not isinstance(input["value"], (int, float)):
                    raise TypeError(
                        "Input {} is not a number, but a {}".format(
                            input_name, type(input["value"])
                        )
                    )
            elif expected_input_type == "string":
                if not isinstance(input["value"], str):
                    raise TypeError(
                        "Input {} is not a string but a {}".format(
                            input_name, type(input["value"])
                        )
                    )
            elif expected_input_type == "array":
                if not isinstance(input["value"], list):
                    raise TypeError(
                        "Input {} is not an array but a {}".format(
                            input_name, type(input["value"])
                        )
                    )

                # Check the subtype
                if expected_list_type == "number":
                    for value in input["value"]:
                        if not isinstance(value, (int, float)):
                            raise TypeError(
                                "Input {} is not an array of numbers but of {}".format(
                                    input_name, type(value)
                                )
                            )
                elif expected_list_type == "string":
                    for value in input["value"]:
                        if not isinstance(value, str):
                            raise TypeError(
                                "Input {} is not an array of strings but of {}".format(
                                    input_name, type(value)
                                )
                            )

            # Return the value
            return input["value"]

    raise TypeError("Input {} not found in inputs".format(input_name))



from abc import ABC, abstractmethod, abstractproperty


class DataProvider(ABC):
    # Data
    @abstractproperty
    def name(self):
        pass

    @abstractproperty
    def is_alive(self):
        return False

    @abstractproperty
    def type(self):
        pass

    # Info
    @abstractmethod
    def get_info(self):
        pass

    # Projects
    @abstractmethod
    def get_projects(self):
        pass

    @abstractmethod
    def get_project(self, id):
        pass

    @abstractmethod
    def delete_project(self, _id):
        pass

    # Samples
    @abstractmethod
    def get_id_list(self, _projectId, _analysis, _from, _to):
        pass

    @abstractmethod
    def get_samples(self, _projectId, _analysis, id_list):
        pass

    # Selections
    @abstractmethod
    def get_selections(self):
        pass

    @abstractmethod
    def get_selection_id_list(self, id):
        pass

    @abstractmethod
    def create_selection(self, name, id_list):
        pass

    @abstractmethod
    def delete_selection(self, id):
        pass

    # Models
    @abstractmethod
    def get_models(self):
        pass

    @abstractmethod
    def get_model_results_id_list(self):
        pass

    @abstractmethod
    def get_model_results(self, id_list):
        pass

    @abstractmethod
    def delete_model(self, id):
        pass



#############################################################################
# Imports
#############################################################################

import debiaiServer.modules.dataProviders.dataProviderManager as data_provider_manager
from debiaiServer.modules.dataProviders.DataProviderException import DataProviderException

#############################################################################
# Selections Management
#############################################################################


def get_selections(dataProviderId, projectId):
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)
        return data_provider.get_selections(projectId), 200
    except DataProviderException as e:
        return e.message, e.status_code


def get_selection_id_list(dataProviderId, projectId, selectionId):
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)
        return data_provider.get_selection_id_list(projectId, selectionId), 200
    except DataProviderException as e:
        return e.message, e.status_code


def post_selection(dataProviderId, projectId, data):
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)
        data_provider.create_selection(
            projectId,
            data["selectionName"],
            data["sampleHashList"],
            data["requestId"] if "requestId" in data else None,
        )
        return "Selection added", 200
    except DataProviderException as e:
        return e.message, e.status_code


def delete_selection(dataProviderId, projectId, selectionId):
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)
        data_provider.delete_selection(projectId, selectionId)
        return "Selection deleted", 200
    except DataProviderException as e:
        return e.message, e.status_code



import requests
import ujson as json

appUrl = "http://localhost:3000/"
testLayoutId = None


def delete_layout(id):
    url = appUrl + "app/layouts/" + id
    resp = requests.request("DELETE", url, headers={}, data={})
    assert resp.status_code == 204


def test_get_layouts():
    url = appUrl + "app/layouts"
    resp = requests.request("GET", url, headers={}, data={})
    layouts = json.loads(resp.text)
    print(layouts)
    assert resp.status_code == 200
    assert type(layouts) is list

    # Remove all layouts
    for layout in layouts:
        assert type(layout) is dict
        assert "id" in layout
        delete_layout(layout["id"])


def test_add_layout():
    global testLayoutId
    url = appUrl + "app/layouts/"
    data = {
        "name": "testName",
        "description": "testDescription",
        "projectId": "testProjectId",
        "dataProviderId": "testDataProviderId",
        "layout": [
            {
                "x": 0,
                "y": 0,
                "width": 1,
                "height": 1,
                "widgetKey": "testWidgetKey",
                "config": {},
                "name": "testName",
            }
        ],
        "selectedColorColumn": "TestColorColumn",
    }
    resp = requests.request("POST", url, headers={}, json=data)
    assert resp.status_code == 204

    # Check if the layout was added
    resp = requests.request("GET", url, headers={}, data={})
    layouts = json.loads(resp.text)
    assert resp.status_code == 200
    assert type(layouts) is list
    assert len(layouts) == 1
    assert layouts[0]["name"] == data["name"]
    assert layouts[0]["description"] == data["description"]
    assert layouts[0]["projectId"] == data["projectId"]
    assert layouts[0]["dataProviderId"] == data["dataProviderId"]
    assert type(layouts[0]["layout"]) is list
    assert len(layouts[0]["layout"]) == 1
    assert layouts[0]["layout"][0]["x"] == data["layout"][0]["x"]
    assert layouts[0]["layout"][0]["y"] == data["layout"][0]["y"]
    assert layouts[0]["selectedColorColumn"] == data["selectedColorColumn"]

    assert "id" in layouts[0]
    assert type(layouts[0]["id"]) is str
    assert len(layouts[0]["id"]) > 0
    testLayoutId = layouts[0]["id"]


def test_delete_layout():
    # Remove the layout
    delete_layout(testLayoutId)

    # Check if the layout was removed
    url = appUrl + "app/layouts/"
    resp = requests.request("GET", url, headers={}, data={})
    layouts = json.loads(resp.text)
    assert resp.status_code == 200
    assert type(layouts) is list
    assert len(layouts) == 0


def test_last_layout_saved():
    # if lastLayoutSaved is true, the previous layout with lastLayoutSaved = true
    # should be deleted

    # Add the first layout
    url = appUrl + "app/layouts/"
    data = {
        "name": "testName",
        "description": "testDescription",
        "projectId": "testProjectId",
        "dataProviderId": "testDataProviderId",
        "lastLayoutSaved": True,
        "layout": [],
    }
    resp = requests.request("POST", url, headers={}, json=data)
    assert resp.status_code == 204

    # Add the second layout
    data = {
        "name": "testName2",
        "description": "testDescription",
        "projectId": "testProjectId",
        "dataProviderId": "testDataProviderId",
        "lastLayoutSaved": True,
        "layout": [],
    }
    resp = requests.request("POST", url, headers={}, json=data)
    assert resp.status_code == 204

    # Check if the first layout was removed
    url = appUrl + "app/layouts/"
    resp = requests.request("GET", url, headers={}, data={})
    layouts = json.loads(resp.text)
    assert resp.status_code == 200
    assert type(layouts) is list
    assert len(layouts) == 1
    assert layouts[0]["lastLayoutSaved"] is True
    assert layouts[0]["name"] == data["name"]
    assert layouts[0]["description"] == data["description"]
    assert layouts[0]["projectId"] == data["projectId"]
    assert layouts[0]["dataProviderId"] == data["dataProviderId"]

    # Remove the layout
    delete_layout(layouts[0]["id"])
